@@ -10,7 +10,7 @@ Supported type annotations:
     List[X], Tuple[X, ...], Set[X], Dict[str, X],
     JSONAble (nested)
 """
-
+import enum
 from dataclasses import dataclass
 from datetime import datetime, timedelta
 from decimal import Decimal
@@ -40,6 +40,11 @@ JSON = Dict[str, V]
 NameConverter = Callable[[str], str]
 
 
+class Action(enum.IntEnum):
+    ENCODING = 1
+    DECODING = 2
+
+
 @dataclass(frozen=True)
 class json_options:
     """Field-level options to override the default conversion behavior.
@@ -50,11 +55,18 @@ class json_options:
     # Uses the name of this field by default.
     name: Optional[str] = None
 
-    # A function that returns a custom key to be mapping in the dictionary.
+    # A function that returns a custom key to be mapping in the dictionary
+    #   whenever converting to dictionary or dataclass.
     # This option is similar to option `name`, but it's a function rather than a string.
     # If the option `name` is set the same time, prefers `name` over `name_converter`.
     name_converter: Optional[NameConverter] = None
 
+    # This option is similar to option `name_converter`,
+    #   but only used when converting dictionary to dataclass.
+    # If this option is not set, default to use `name_converter`.
+    # If the option `name` is set the same time, prefers `name` over `name_inverter`.
+    name_inverter: Optional[NameConverter] = None
+
     # Omit this field if it has an empty value, defaults to False.
     # This option is only about encoding.
     omitempty: Optional[bool] = None
@@ -181,7 +193,7 @@ class JSONAble:
         elif t is float:
             return float
         elif t is Decimal:
-            return Decimal
+            return _decode_decimal
         elif t is datetime:
             return _decode_datetime
         elif t is timedelta:
@@ -292,7 +304,7 @@ class JSONAble:
                     continue
 
             # Key in dictionary `d`.
-            k = _util_get_field_key(name, options)
+            k = _util_get_field_key(name, options, Action.ENCODING)
 
             # Encode.
             encoder = options.encoder or self.get_encoder(t)
@@ -313,7 +325,7 @@ class JSONAble:
             options = cls._get_json_options(f)
 
             # Key in dictionary.
-            k = _util_get_field_key(name, options)
+            k = _util_get_field_key(name, options, Action.DECODING)
 
             if options.skip:
                 continue
@@ -366,6 +378,7 @@ _encode_jsonable = lambda x: x.json()
 _decode_datetime = lambda x: datetime.fromtimestamp(int(x))
 _decode_timedelta = lambda x: timedelta(seconds=int(x))
 _decode_None = lambda _: None
+_decode_decimal = lambda x: Decimal(str(x))
 
 _default_omitempty_tester = lambda x: not x
 
@@ -402,13 +415,18 @@ def _replace_mapping_proxy(m: MappingProxyType, kwds) -> MappingProxyType:
     return MappingProxyType(d)
 
 
-def _util_get_field_key(name: str, options: json_options) -> str:
+def _util_get_field_key(name: str, options: json_options, action: Action) -> str:
     """A util function returns the `key` in target dictionary.
     :param name: the field's name.
     :param options: the json_options for this field.
     """
     if options.name:
         return options.name
-    elif options.name_converter:
+
+    if action == Action.DECODING and options.name_inverter:
+        return options.name_inverter(name)
+
+    if options.name_converter:
         return options.name_converter(name)
+
     return name
